/*
 * Sudokuki - essential sudoku game
 * Copyright (C) 2007-2012 Sylvain Vedrenne
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.jankenpoi.sudokuki.ui.swing;

import static net.jankenpoi.i18n.I18n._;

import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Locale;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.JMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import net.jankenpoi.i18n.I18n;
import net.jankenpoi.i18n.LocaleListener;
import net.jankenpoi.sudokuki.ui.L10nComponent;

@SuppressWarnings("serial")
public class LanguageMenu extends JMenu implements L10nComponent {

        private HashMap<String, JRadioButtonMenuItem> itemsMap = new HashMap<String, JRadioButtonMenuItem>();

        private String langCode;
        
        public LanguageMenu() {
                addItems();
                setIcon(languageIcon(_("DETECTED_LANGUAGE")));
                
                addMenuListener(new MenuListener() {
                        
                        @Override
                        public void menuSelected(MenuEvent arg0) {
                                final String detectedLanguage = _("DETECTED_LANGUAGE");
                                langCode = detectedLanguage;
                        }
                        
                        @Override
                        public void menuDeselected(MenuEvent arg0) {
                                I18n.reset(langCode);
                        }
                        
						@Override
                        public void menuCanceled(MenuEvent arg0) {
                                I18n.reset(langCode);
                        }
                });
                
                final String detectedLanguage = _("DETECTED_LANGUAGE");
                JRadioButtonMenuItem selectedItem = itemsMap.get(detectedLanguage);
                setText(_("Language"));
                if (selectedItem != null) {
                        selectedItem.setSelected(true);
                }
                localeListener = new LocaleListenerImpl(this);
                I18n.addLocaleListener(localeListener);
        }
		        
        private void addItems() {
                ButtonGroup myGroup = new ButtonGroup();
                addItem("de", _("German"), myGroup);
                addItem("el", _("Greek"), myGroup);
                addItem("en", _("English"), myGroup);
                addItem("eo", _("Esperanto"), myGroup);
                addItem("es", _("Spanish"), myGroup);
                addItem("fr", _("French"), myGroup);
                addItem("ja", _("Japanese"), myGroup);
                addItem("pt", _("Portuguese"), myGroup);
                addItem("ru", _("Russian"), myGroup);
                addItem("zh", _("Mandarin"), myGroup);
        }

        private void addItem(final String code, String language, ButtonGroup group) {
                JRadioButtonMenuItem radioItem;

                radioItem = new JRadioButtonMenuItem(language);
                itemsMap.put(code, radioItem);
                radioItem.setAction(new AbstractAction(language, languageIcon(code)) {

                        @Override
                        public void actionPerformed(ActionEvent arg0) {
                                I18n.reset(code);
                        }
                });

                radioItem.addMouseListener(new MouseAdapter() {

                        @Override
                        public void mouseEntered(MouseEvent e) {
                                I18n.reset(code);
                        }
                        
                        @Override
                        public void mouseExited(MouseEvent e) {
                                I18n.reset(langCode);
                        }

                });
                group.add(radioItem);
                add(radioItem);
        }

        private final LocaleListener localeListener;
		@Override
		public void setL10nMessages(Locale locale, String languageCode) {
			setText(_("Language"));
			setIcon(languageIcon(languageCode));
			
			if (this.isSelected()) {
				return;
			}
            refreshItem(languageCode, "de", "German");
            refreshItem(languageCode, "el", "Greek");
            refreshItem(languageCode, "en", "English");
            refreshItem(languageCode, "eo", "Esperanto");
            refreshItem(languageCode, "es", "Spanish");
            refreshItem(languageCode, "fr", "French");
            refreshItem(languageCode, "ja", "Japanese");
            refreshItem(languageCode, "pt", "Portuguese");
            refreshItem(languageCode, "ru", "Russian");
            refreshItem(languageCode, "zh", "Mandarin");
		}

		private void refreshItem(final String langCode, final String itemCode, final String langName) {
			JRadioButtonMenuItem item = itemsMap.get(itemCode);
			if (item != null) {
				item.setText(_(langName));
				if (itemCode.equals(langCode)) {
					item.setSelected(true);
				}
			}
		}
        
		public static Icon languageIcon(final String langCode) {
			if ("de".equals(langCode)) {
				return StockIcons.ICON_FLAG_DE;
			} else if ("el".equals(langCode)) {
				return StockIcons.ICON_FLAG_EL;
			} else if ("eo".equals(langCode)) {
				return StockIcons.ICON_FLAG_EO;
			} else if ("en".equals(langCode)) {
				return StockIcons.ICON_FLAG_EN;
			} else if ("es".equals(langCode)) {
				return StockIcons.ICON_FLAG_ES;
			} else if ("fr".equals(langCode)) {
				return StockIcons.ICON_FLAG_FR;
			} else if ("ja".equals(langCode)) {
				return StockIcons.ICON_FLAG_JA;
			} else if ("pt".equals(langCode)) {
				return StockIcons.ICON_FLAG_PT;
			} else if ("ru".equals(langCode)) {
				return StockIcons.ICON_FLAG_RU;
			} else if ("zh".equals(langCode)) {
				return StockIcons.ICON_FLAG_ZH;
			}
			return StockIcons.ICON_GO_HOME;
		}
		
}
